
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include "iglu_arco.h"

namespace iglu{

  using namespace std;

IgluArco::IgluArco(double x0,double y0,double radio_x,double radio_y,
		   double angulo_inicial,double angulo_final,IgluColor c) :
  IgluObjetoColor(c,IGLU_ARC_ELLIPSE),x(x0),y(y0),rx(radio_x),ry(radio_y),
  theta_i(angulo_inicial),theta_f(angulo_final){
  /* Control angulos */
  if (angulo_inicial>angulo_final){
    cout << "IgluArco: angulo inicial debe ser menor que angulo final" << endl;
    exit(1);
  }
  /* Fin control angulos */
}

IgluArco::~IgluArco(){
}

double IgluArco::x_center() const{
  return x;
}

double IgluArco::y_center() const{
  return y;
}

double IgluArco::x_radius() const{
  return rx;
}

double IgluArco::y_radius() const{
  return ry;
}

double IgluArco::angle_start() const{
  return theta_i;
}

double IgluArco::angle_end() const{
  return theta_f;
}

IgluArcoElipse::IgluArcoElipse(double x0,double y0,double radio_x,
			       double radio_y,double angulo_inicial,
			       double angulo_final,IgluColor c) 
  : IgluArco(x0,y0,radio_x,radio_y,angulo_inicial,angulo_final,c){
  /* Control angulos */
  if (angulo_inicial>angulo_final){
    cout << "IgluArco: angulo inicial debe ser menor que angulo final" << endl;
    exit(1);
  }
  /* Fin control angulos */
}

IgluArcoElipse::~IgluArcoElipse(){
}

void IgluArcoElipse::report_status(){
  cout << (*this) << endl;
}

ostream & operator << (ostream & os, const IgluArcoElipse & a){
  os << "Arco elipse : (" << a.x_center() << "," << a.y_center() << "), " << 
    a.x_radius() << ", " << a.y_radius() << ", " << 
    a.angle_start() << " -> " << a.angle_end() << "; " << a.color();
  return os;
}

IgluArcoElipseLleno::IgluArcoElipseLleno(double x0,double y0,
					 double radio_x,double radio_y,
					 double angulo_inicial,
					 double angulo_final,
					 IgluColor c) : 
  IgluArcoElipse(x0,y0,radio_x,radio_y,angulo_inicial,angulo_final,c){
  tipo_objeto = IGLU_FILLED_ARC_ELLIPSE;
}

IgluArcoElipseLleno::~IgluArcoElipseLleno(){
}

IgluArcoCirculo::IgluArcoCirculo(double x0,double y0,double radio,
				 double angulo_inicial,double angulo_final,
				 IgluColor c) : IgluArco(x0,y0,
							 radio,radio,
							 angulo_inicial,
							 angulo_final,c){
}

IgluArcoCirculo::~IgluArcoCirculo(){
}

double IgluArcoCirculo::radius() const{
  return rx;
}


/*void IgluArcoCirculo::plot(IgluDriver d){
  d.plot_arc_circle(x,y,r,theta_i,theta_f,mycolor);
  }*/

IgluArcoCirculoLleno::IgluArcoCirculoLleno(double x0,double y0,double radio,
					   double angulo_inicial,
					   double angulo_final,
					   IgluColor c) : 
  IgluArcoElipseLleno(x0,y0,radio,radio,angulo_inicial,angulo_final,c){
}

IgluArcoCirculoLleno::~IgluArcoCirculoLleno(){
}

}
