
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include "iglu_dibujo.h"

namespace iglu{

  using namespace std;

IgluDibujo::IgluDibujo(){
  variables_default();
  driver.set_variables(ancho_default_pixeles,alto_default_pixeles);	       
}

IgluDibujo::IgluDibujo(int ancho_pixeles,int alto_pixeles){
  variables_default();
  ancho_default_pixeles = ancho_pixeles;
  alto_default_pixeles = alto_pixeles;
  driver.set_variables(ancho_pixeles,alto_pixeles);
//   xiP = 0;
//   xfP = ancho_pixeles;
//   yfP = 0;
//   yiP = alto_pixeles;
//   driver.set_variables(xiP,xfP,yiP,yfP,xiR,xfR,yiR,yfR,nombre);
}

IgluDibujo::IgluDibujo(int ancho_pixeles,int alto_pixeles,string titulo){
  variables_default();
  ancho_default_pixeles = ancho_pixeles;
  alto_default_pixeles = alto_pixeles;
  driver.set_variables(ancho_pixeles,alto_pixeles,titulo);
//   nombre = titulo;
//   xiP = 0;
//   xfP = ancho_pixeles;
//   yfP = 0;
//   yiP = alto_pixeles;
//   driver.set_variables(xiP,xfP,yiP,yfP,xiR,xfR,yiR,yfR,nombre);
}

IgluDibujo::IgluDibujo(string titulo){
  variables_default();
  driver.set_variables(ancho_default_pixeles,alto_default_pixeles,titulo);
//   nombre = titulo;
//   driver.set_variables(xiP,xfP,yiP,yfP,xiR,xfR,yiR,yfR,nombre);
}

IgluDibujo::~IgluDibujo(){
  
}

void IgluDibujo::variables_default(){
  ancho_default_pixeles = 500;
  alto_default_pixeles = 500;
  initialize_layers();
  //  driver = new IgluDriverX();

//   xiP = 0;
//   xfP = 500;
//   yfP = 0;
//   yiP = 500;
//   nombre = "";
//   xiR = xiP;
//   xfR = xfP;
//   yiR = yfP;
//   yfR = yiP;
}

void IgluDibujo::initialize_layers(){
  layers = 1;
  current_layer = 1;
  lista_objetos.resize(1);
  vacio = 1;
}
  
void IgluDibujo::wait(){
  flush();
  driver.wait(lista_objetos);
}

void IgluDibujo::wait(double tval){
  flush();
  driver.wait(lista_objetos,tval);
  //  usleep(int(1000000*tval));
}

void IgluDibujo::plot_line(double x1,double y1,double x2,double y2){
  IgluObjetoColor * l = new IgluLinea(x1,y1,x2,y2,driver.foreground_color());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(l);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),l);
  /*
  IgluLinea l(x1,y1,x2,y2,driver.foreground_color());
  lista_lineas.push_front(l);*/
  driver.plot_line(x1,y1,x2,y2,driver.foreground_color());
} 

void IgluDibujo::plot_line(double x[], double y[], int n){
  for (int i=0;i<n-1;i++){
    plot_line(x[i],y[i],x[i+1],y[i+1]);
  }
}

void IgluDibujo::plot_line(int x[], int y[], int n){
  for (int i=0;i<n-1;i++){
    plot_line(x[i],y[i],x[i+1],y[i+1]);
  }
}

void IgluDibujo::plot_line(vector<double> x,vector<double> y){
  if (x.size()==y.size()){
    for (unsigned int i=0;i<(x.size()-1);i++){
      plot_line(x[i],y[i],x[i+1],y[i+1]);
    }
  }
  else {
    cout << "plot_line(x,y): vectores deben ser de igual longitud" << endl;
  }
}

void IgluDibujo::plot_line(vector<int> x,vector<int> y){
  if (x.size()==y.size()){
    for (unsigned int i=0;i<(x.size()-1);i++){
      plot_line(x[i],y[i],x[i+1],y[i+1]);
    }
  }
  else {
    cout << "plot_line(x,y): vectores deben ser de igual longitud" << endl;
  }
}

void IgluDibujo::plot_point(double x,double y){
  IgluObjetoColor * p = new IgluPunto(x,y,driver.foreground_color());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(p);
  //lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),p);
  /*
  IgluPunto p(x,y,driver.foreground_color());
  lista_puntos.push_front(p);
  */
  driver.plot_point(x,y,driver.foreground_color());
}

void IgluDibujo::plot_point(double x[],double y[],int n){
 for (int i=0;i<n;i++) plot_point(x[i],y[i]);
}

void IgluDibujo::plot_point(int x[],int y[],int n){
 for (int i=0;i<n;i++) plot_point(x[i],y[i]);
}

void IgluDibujo::plot_point(vector<double> x,vector<double> y){
  if (x.size()==y.size()){
    for (unsigned int i=0;i<x.size();i++) plot_point(x[i],y[i]);
  }
  else {
    cout << "plot_point(x,y): vectores deben ser de igual longitud" << endl;
  }    
}

void IgluDibujo::plot_point(vector<int> x,vector<int> y){
  if (x.size()==y.size()){
    for (unsigned int i=0;i<x.size();i++) plot_point(x[i],y[i]);
  }
  else {
    cout << "plot_point(x,y): vectores deben ser de igual longitud" << endl;
  }    
}

void IgluDibujo::plot_rectangle(double x1,double y1,double x2,double y2){
  IgluObjetoColor * r = 
    new IgluRectangulo(x1,y1,x2,y2,driver.foreground_color()); 
  vacio = 0;
  lista_objetos[current_layer-1].push_back(r);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),r);
  //  lista_rectangulos.push_front(IgluRectangulo(x1,y1,x2,y2,driver.foreground_color()));
  driver.plot_rectangle(x1,y1,x2,y2,driver.foreground_color());
}

void IgluDibujo::plot_filled_rectangle(double x1,double y1,
				       double x2,double y2){
  IgluObjetoColor * r = 
    new IgluRectanguloLleno(x1,y1,x2,y2,driver.foreground_color()); 
  vacio = 0;
  lista_objetos[current_layer-1].push_back(r);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),r);
  /*
  lista_rectangulos_llenos.push_front(IgluRectanguloLleno(x1,y1,x2,y2,
							  driver.foreground_color()));
  */
  driver.plot_filled_rectangle(x1,y1,x2,y2,driver.foreground_color());
}

void IgluDibujo::plot_square(double x,double y,double a){
  /* (x,y) coordenadas del centro
     a     arista  
  */
  plot_rectangle(x-a/2,y-a/2,x+a/2,y+a/2);
}

void IgluDibujo::plot_filled_square(double x,double y,double a){
  /* (x,y) coordenadas del centro
     a     arista  
  */
  plot_filled_rectangle(x-a/2,y-a/2,x+a/2,y+a/2);
}

void IgluDibujo::plot_arc_circle(double x,double y,double r,
		       double theta_i,double theta_f){
  plot_arc_ellipse(x,y,r,r,theta_i,theta_f);

  /* Version original, cuando solo habia circulos */
//   IgluObjetoColor * c =
//     new IgluArcoCirculo(x,y,r,theta_i,theta_f,driver.foreground_color());
//   vacio = 0;
//   lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
//   driver.plot_arc_circle(x,y,r,theta_i,theta_f,driver.foreground_color());
}

void IgluDibujo::plot_circle(double x,double y,double r){
  plot_arc_ellipse(x,y,r,r,0,360);
}

void IgluDibujo::plot_filled_arc_circle(double x,double y,double r,
					double theta_i,double theta_f){
  plot_filled_arc_ellipse(x,y,r,r,theta_i,theta_f);
}
  
void IgluDibujo::plot_filled_circle(double x,double y,double r){
  plot_filled_arc_ellipse(x,y,r,r,0,360);
}

void IgluDibujo::plot_arc_ellipse(double x,double y,double radio_x,
			      double radio_y,double theta_i,double theta_f){
  IgluObjetoColor * c = 
    new IgluArcoElipse(x,y,radio_x,radio_y,theta_i,theta_f,
		       driver.foreground_color());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(c);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
  driver.plot_arc_ellipse(x,y,radio_x,radio_y,theta_i,theta_f,
			  driver.foreground_color()); 
}

void IgluDibujo::plot_filled_arc_ellipse(double x,double y,
					 double radio_x,double radio_y,
					 double theta_i,double theta_f){
  IgluObjetoColor * c =
    new IgluArcoElipseLleno(x,y,radio_x,radio_y,theta_i,theta_f,
			    driver.foreground_color());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(c);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
  driver.plot_filled_arc_ellipse(x,y,radio_x,radio_y,theta_i,theta_f,
				 driver.foreground_color()); 
}

void IgluDibujo::plot_ellipse(double x,double y,double radio_x,
			      double radio_y){
  plot_arc_ellipse(x,y,radio_x,radio_y,0,360);
}

void IgluDibujo::plot_filled_ellipse(double x,double y,double radio_x,
				     double radio_y){
  plot_filled_arc_ellipse(x,y,radio_x,radio_y,0,360);
}

void IgluDibujo::plot_fake_arc_circle(double x,double y,double radio,
				       double theta_i,double theta_f,
				       string eje){
  vector<double> radios = calcular_radios(radio,eje);
  plot_arc_ellipse(x,y,radios[0],radios[1],theta_i,theta_f);
}

void IgluDibujo::plot_fake_circle(double x,double y,double radio,string eje){
  plot_fake_arc_circle(x,y,radio,0,360,eje);
}

void IgluDibujo::plot_fake_filled_arc_circle(double x,double y,double radio,
					     double theta_i,double theta_f,
					     string eje){
  vector<double> radios = calcular_radios(radio,eje);
  plot_filled_arc_ellipse(x,y,radios[0],radios[1],theta_i,theta_f);
}

void IgluDibujo::plot_fake_filled_circle(double x,double y,
					 double radio,string eje){
  plot_fake_filled_arc_circle(x,y,radio,0,360,eje);
}

void IgluDibujo::plot_text(double x,double y,string s,string etiqueta){
  IgluObjetoColor * c = new IgluTexto(x,y,s,driver.foreground_color(),
				      etiqueta,driver.get_scale_font());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(c);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
  driver.plot_text(x,y,s,driver.foreground_color(),driver.get_scale_font());
}

void IgluDibujo::plot_centered_text(double x,double y,
				    string s,string etiqueta){
  IgluObjetoColor * c = new IgluTextoCentrado(x,y,s,driver.foreground_color(),
					      etiqueta,
					      driver.get_scale_font());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(c);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
  driver.plot_centered_text(x,y,s,driver.foreground_color(),
			    driver.get_scale_font());
}

void IgluDibujo::clip_rectangle(double xmin, double ymin, 
				double xmax, double ymax){
  IgluObjetoColor * c = new IgluClipRectangulo(xmin,ymin,xmax,ymax,
					       driver.background_color());
  vacio = 0;
  lista_objetos[current_layer-1].push_back(c);
  //  lista_objetos[current_layer-1].insert_after(lista_objetos[current_layer-1].previous(lista_objetos[current_layer-1].end()),c);
  driver.plot_clip_rectangle(xmin,ymin,xmax,ymax,driver.background_color());
}

void IgluDibujo::map_coordinates(double xmin,double xmax,double ymin,double ymax){
  if (vacio){
    driver.map_coordinates(xmin,xmax,ymin,ymax);
  }
  else {
    cout << "Iglu: map_coordinates() solo puede usarse antes de dibujar "
	 << "el primer objeto.\n" 
	 << "      Ignorando las nuevas coordenadas." << endl;
  }
}

void IgluDibujo::set_layers(int n){
  if (layers<=n){
    layers = n;
    lista_objetos.resize(n);
  }
  else {
    cout << "IgluDibujo::set_layers(n): No se puede reducir el numero de "
	 << "capas. Ignorando." << endl;
  }
}

void IgluDibujo::use_layer(int n){
  if (layers>=n){
    current_layer = n;
  }
  else {
    cout << "IgluDibujo::use_layer(n): No se puede usar capa inexistente. "
	 << "Ignorando." << endl;
  }
}

void IgluDibujo::clear_layer(int n){
  //  cout << "Limpiando layer" << n << endl;
  if (layers>=n){
    lista_objetos[n-1].clear();
    driver.clean();
    
  }
  else {
    cout << "IgluDibujo::use_layer(n): No se puede usar capa inexistente. "
	 << "Ignorando." << endl;
  }
}

void IgluDibujo::flush(){ 
  driver.flush();
}

void IgluDibujo::report_status(){
  cout << "Objeto: IgluDibujo" << endl;
  cout << "   nombre = " << driver.nombre_dibujo() << endl;
  cout << "   [xmin,xmax,ymin,ymax] (pixeles) = " <<
    "[" << driver.xmin_pixeles() << "," << driver.xmax_pixeles()
       << "," << driver.ymin_pixeles() << "," << driver.ymax_pixeles()
       << "]" <<endl;
  cout << "   [xmin,xmax,ymin,ymax] (reales) = " << 
    "[" << driver.xmin_reales() << "," << driver.xmax_reales()
       << "," << driver.ymin_reales() << "," << driver.ymax_reales()
       << "]" <<endl;
  cout << "Capas: " << layers << ". Capa actual: " << current_layer 
       << "." << endl;

  for (unsigned int i=0;i<lista_objetos.size();i++){
    cout << "   Capa: " << i+1 << endl;
    cout << "     Numero de objetos: " << lista_objetos[i].size() << endl;
    cout << "     Objetos: " << endl;
    for (list<IgluObjetoColor *>::iterator it = lista_objetos[i].begin();
	 it != lista_objetos[i].end(); it++){
      (*it)->report_status();
    }
  }
  
  /*  copy(lista_objetos.begin(),lista_objetos.end(),
      ostream_iterator<IgluObjetoColor *>(cout,"\n"));*/
  /*
  cout << "   Puntos: " << endl;
  copy(lista_puntos.begin(),lista_puntos.end(),
       ostream_iterator<IgluPunto>(cout,"\n"));
  cout << "   Lineas: " << endl;
  copy(lista_lineas.begin(),lista_lineas.end(),
       ostream_iterator<IgluLinea>(cout,"\n"));
  cout << "   Rectangulos: " << endl;
  copy(lista_rectangulos.begin(),lista_rectangulos.end(),
       ostream_iterator<IgluRectangulo>(cout,"\n"));
  cout << "   Rectangulos Llenos: " << endl;
  copy(lista_rectangulos_llenos.begin(),lista_rectangulos_llenos.end(),
       ostream_iterator<IgluRectanguloLleno>(cout,"\n"));
  cout << "   Arco Circulos: " << endl;
  copy(lista_arco_circulos.begin(),lista_arco_circulos.end(),
       ostream_iterator<IgluArcoCirculo>(cout,"\n"));
  cout << "   Arco Circulo Llenos: " << endl;
  copy(lista_arco_circulos_llenos.begin(),lista_arco_circulos_llenos.end(),
       ostream_iterator<IgluArcoCirculoLleno>(cout,"\n"));
  */
 
}


void IgluDibujo::clean(){
  lista_objetos.clear();
  initialize_layers();
  driver.clean();
}

void IgluDibujo::erase_label(string s){
  list<IgluObjetoColor *>::iterator it_found;
  int found = 0;
  for (unsigned int i=0;i<lista_objetos.size();i++){
    for (list<IgluObjetoColor *>::iterator it = lista_objetos[i].begin();
	 it != lista_objetos[i].end(); it++){ 
      if ((*it)->label()==s){
	it_found = it;
	found = 1;
	break;
      }
      
    }
    //    cout << "erase_label: " << s << " " << i << " " << found << endl;
    driver.clean();
    if (found){
      lista_objetos[i].erase(it_found);
      found = 0;
    }
  }
}

void IgluDibujo::save_base(string s, int warn){
  IgluDriverPostScript driver_ps(warn);
  //  driver_ps.set_variables(ancho_default_pixeles,alto_default_pixeles,s); 
  driver_ps.bounding_box(ancho_default_pixeles,alto_default_pixeles);
  //  cout << ancho_default_pixeles << ", " << alto_default_pixeles << endl;
  driver_ps.set_variables(driver.xmax_pixeles(),driver.ymin_pixeles(),s);
  driver_ps.map_coordinates(driver.xmin_reales(),driver.xmax_reales(),
			    driver.ymin_reales(),driver.ymax_reales());
  driver_ps.replot(lista_objetos);
}

void IgluDibujo::save(string s){
  save_base(s,0);
}

void IgluDibujo::save_safe(string s){
  save_base(s,1);
}

//////////////////////////////////////////
// HANDLES COLORS
//////////////////////////////////////////


// void IgluDibujo::color_set(double r1,double g1, double b1){
//   colorpixel = colorbase + 
//     (unsigned long)(colorblu*b1) +
//     (unsigned long)(colorblu*g1+1)*(colorblu+1) + 
//     (unsigned long)(colorblu*r1+1)*(colorgre+colorblu+1);
//   XSetForeground(mydisplay, mygc, colorpixel);
// }
void IgluDibujo::color_set(double r1,double g1,double b1){
  driver.color_set(r1,g1,b1);
}

void IgluDibujo::color_set(int i){
  driver.color_set(i);
}

void IgluDibujo::color_set(IgluColor c){
  driver.color_set(c.red(),c.green(),c.blue());
}

void IgluDibujo::scale_font(double f){
  driver.scale_font(f);
}

void IgluDibujo::replot(){
  driver.replot(lista_objetos);
}

vector<double> IgluDibujo::calcular_radios(double radio,string eje){
  vector<double> radios(2);
  double radio_x=0, radio_y=0;
  if (eje=="x"){
    radio_x=radio;
    radio_y=radio*(double(ancho_default_pixeles)/double(alto_default_pixeles))*
      (driver.anchoyR()/driver.anchoxR());
  }
  else if (eje=="y"){
    radio_x=radio*(double(alto_default_pixeles)/double(ancho_default_pixeles))*
      (driver.anchoxR()/driver.anchoyR());
    radio_y=radio;
  }
  else{
    cout << 
      "IgluDibujo::calcular_radios(double,string) : " << endl
	 << "    string debe ser \"x\" o \"y\". Usando radio=0" << endl;
  }

  radios[0]=radio_x;
  radios[1]=radio_y;
  return radios;
}

}
