
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#ifndef IGLU_DIBUJO
#define IGLU_DIBUJO

#include <string>
#include <list>
#include <vector>
#include "iglu_linea.h"
#include "iglu_punto.h"
#include "iglu_rectangulo.h"
#include "iglu_arco.h"
#include "iglu_driver.h"
#include "iglu_texto.h"
#include "iglu_clip.h"
#include "iglu_color.h"

namespace iglu{

  using namespace std;

class IgluDibujo{
 protected:
  int ancho_default_pixeles;
  int alto_default_pixeles;
  int layers;
  int current_layer;
  int vacio;

  vector<list<IgluObjetoColor *> > lista_objetos;

  /*
  list<IgluLinea> lista_lineas;
  list<IgluPunto> lista_puntos;
  list<IgluRectangulo> lista_rectangulos;
  list<IgluRectanguloLleno> lista_rectangulos_llenos;
  list<IgluArcoCirculo> lista_arco_circulos;
  list<IgluArcoCirculoLleno> lista_arco_circulos_llenos;
  */
  IgluDriverX  driver;

  void variables_default();
  void initialize_layers();
  void save_base(string,int);
  vector<double> calcular_radios(double,string);

 public:
  IgluDibujo();
  IgluDibujo(int,int);
  IgluDibujo(int,int,string);
  IgluDibujo(string);
  ~IgluDibujo();

  void plot_line(double,double,double,double);
  void plot_line(double [],double [],int);
  void plot_line(int [],int [],int);
  void plot_line(vector<double>,vector<double>);
  void plot_line(vector<int>,vector<int>);
  void plot_point(double,double);
  void plot_point(double [],double [],int);
  void plot_point(int [],int [],int);
  void plot_point(vector<double>,vector<double>);
  void plot_point(vector<int>,vector<int>);
  void plot_rectangle(double,double,double,double);
  void plot_filled_rectangle(double,double,double,double);
  void plot_square(double,double,double);
  void plot_filled_square(double,double,double);
  void plot_arc_circle(double,double,double,double,double);
  void plot_circle(double,double,double);
  void plot_filled_arc_circle(double,double,double,double,double);
  void plot_filled_circle(double,double,double);
  void plot_arc_ellipse(double,double,double,double,double,double);
  void plot_ellipse(double,double,double,double);
  void plot_filled_arc_ellipse(double,double,double,double,double,double);
  void plot_filled_ellipse(double,double,double,double);
  void plot_fake_arc_circle(double,double,double,double,double,string);
  void plot_fake_circle(double,double,double,string);
  void plot_fake_filled_arc_circle(double,double,double,double,double,string);
  void plot_fake_filled_circle(double,double,double,string);
  void plot_text(double,double,string,string="");
  void plot_centered_text(double,double,string,string="");
  void clip_rectangle(double,double,double,double);

  void map_coordinates(double,double,double,double);
  void color_set(double,double,double);
  void color_set(int);
  void color_set(IgluColor);
  void scale_font(double);
  void set_layers(int);
  void use_layer(int);
  void clear_layer(int);

  void wait();
  void wait(double);
  void flush();
  void report_status();
  void clean();
  void erase_label(string);
  void replot();
  void save(string);
  void save_safe(string);
};

}

#endif
