
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include "iglu_driver.h"

namespace iglu{

  using namespace std;

IgluDriver::IgluDriver(){
}

IgluDriver::~IgluDriver(){
}

void IgluDriver::map_coordinates(double xmin,double xmax,double ymin,double ymax){
  xiR = xmin;
  xfR = xmax;
  yiR = ymin;
  yfR = ymax;
}

IgluColor IgluDriver::foreground_color(){
  return current_color;
}

IgluColor IgluDriver::background_color(){
  return current_background_color;
}

double IgluDriver::anchoxR(){
  return xfR-xiR;
}

double IgluDriver::anchoyR(){
  return yfR-yiR;
}

void IgluDriver::color_set(double r1,double g1,double b1){
  current_color = IgluColor(r1,g1,b1);
  color_set_this_driver(r1,g1,b1);
}

void IgluDriver::color_set(IgluColor c){
  color_set(c.red(),c.green(),c.blue());
}

void IgluDriver::scale_font(double f){
  cout << "IgluDriver::scale_font:" << f << endl;
  current_scale_font = f;
  scale_font_this_driver(f);
}

double IgluDriver::get_scale_font(){
  return current_scale_font;
}

double IgluDriver::xmin_reales(){
  return xiR;
}

double IgluDriver::xmax_reales(){
  return xfR;
}

double IgluDriver::ymin_reales(){
  return yiR;
}

double IgluDriver::ymax_reales(){
  return yfR;
}

string IgluDriver::nombre_dibujo(){
  return nombre;
}

void IgluDriver::replot(list<IgluObjetoColor *>  l){
  int tipo;
  for (list<IgluObjetoColor *>::iterator it = l.begin(); it != l.end(); 
       it++){
    tipo = (*it)->tipo();
    switch (tipo){
    case IGLU_LINE:
      //      IgluLinea * l = *it;
      /*      cout << "Dibujando linea" << endl;
	      cout << (*it)->x_end() << " " << (*it)->y_end() << endl;*/
      plot_line((*it)->x_start(),(*it)->y_start(),(*it)->x_end(),
		(*it)->y_end(), (*it)->color());
      break;
    case IGLU_POINT:
      //      cout << "Dibujando punto" << endl;
      plot_point((*it)->x(),(*it)->y(),(*it)->color());
      break;
    case IGLU_RECTANGLE:
      //      cout << "Dibujando rectangulo" << endl;
      plot_rectangle((*it)->x_start(),(*it)->y_start(),(*it)->x_end(),
		     (*it)->y_end(), (*it)->color());
      break;
    case IGLU_FILLED_RECTANGLE:
      //      cout << "Dibujando rectangulo lleno" << endl;
      plot_filled_rectangle((*it)->x_start(),(*it)->y_start(),(*it)->x_end(),
			    (*it)->y_end(),(*it)->color());
      break;
    case IGLU_ARC_ELLIPSE:
      //      cout << "Dibujando arco" << endl;
      plot_arc_ellipse((*it)->x_center(),(*it)->y_center(),(*it)->x_radius(),
		      (*it)->y_radius(),
		      (*it)->angle_start(),(*it)->angle_end(),(*it)->color());
      break;
     case IGLU_FILLED_ARC_ELLIPSE:
      //      cout << "Dibujando arco lleno" << endl;
      plot_filled_arc_ellipse((*it)->x_center(),(*it)->y_center(),
			     (*it)->x_radius(), (*it)->y_radius(),
			      (*it)->angle_start(),
			     (*it)->angle_end(),(*it)->color());
      break;
   case IGLU_TEXT:
     plot_text((*it)->x(),(*it)->y(),(*it)->text(),(*it)->color(),(*it)->scale());
      break;
    case IGLU_CENTERED_TEXT:
      plot_centered_text((*it)->x(),(*it)->y(),(*it)->text(),(*it)->color(),(*it)->scale());
      break;
    case IGLU_CLIP_RECTANGLE:
       plot_clip_rectangle((*it)->x_start(),(*it)->y_start(),
 			  (*it)->x_end(),(*it)->y_end(),(*it)->color());
      break;
    }
  }
}

void IgluDriver::replot(vector<list<IgluObjetoColor *> > l){
  for (unsigned int i=0;i<l.size();i++){
    IgluDriver::replot(l[i]);
  }
}

/*void IgluDriver::plot_line(IgluLinea * l){
  plot_line(l.x_start(),l.y_start(),l.x_end(),l.y_end(),l.y_color());
  }*/

IgluDriverX::IgluDriverX(){
}

IgluDriverX::~IgluDriverX(){
  XFreeGC(mydisplay, mygc) ;
  XDestroyWindow(mydisplay, mywindow) ;
  XCloseDisplay(mydisplay) ;
}

void IgluDriverX::set_variables(int ancho_pixeles,int alto_pixeles,string n){
  variables_default();
  ancho_default = ancho_pixeles;
  alto_default = alto_pixeles;
  xiP = 0;
  xfP = ancho_pixeles;
  yiP = alto_pixeles;
  yfP = 0;
  xiR = xiP;
  xfR = xfP;
  yiR = yfP;
  yfR = yiP;
  nombre = n;
  crear_dibujo();
}

void IgluDriverX::variables_default(){
  xpos = 5;
  ypos = 500;
  border_width  = 5;
  current_color = IgluColor(0,0,0);
  current_background_color = IgluColor(1,1,1);
  nombre = "";
}

void IgluDriverX::crear_dibujo(){

  // initialize window, display, screen
  // ----------------------------------

  /* Version JR
     mydisplay=XOpenDisplay("") ;*/
  
  /* Version VM */
  char *display_name = getenv("DISPLAY");  /* address of the X display. */
  mydisplay = XOpenDisplay(display_name);

  if(mydisplay ==NULL){
      cout <<"*** can't connect to server"<< endl ;
      exit(0) ;
  }
  myscreen=DefaultScreen(mydisplay) ;


  foreground = BlackPixel(mydisplay,myscreen);
  background = WhitePixel(mydisplay,myscreen);  

  mywindow = XCreateSimpleWindow( mydisplay,
				  DefaultRootWindow(mydisplay), 
				  xpos, ypos, anchoxP(), anchoyP(),
				  border_width, foreground, background) ;

  /* Version VM */
  XMapWindow(mydisplay, mywindow);

  

  /* Opcion obsoleta en XFree 4
  // redraw window when in focus
  // ---------------------------
  XSetWindowAttributes setwinattr ;
  unsigned long valuemask;
  valuemask =CWBackingStore ;
  setwinattr.backing_store=Always ;  
  XChangeWindowAttributes(mydisplay, mywindow, valuemask, &setwinattr);
  */

  XStoreName(mydisplay, mywindow, nombre.c_str()) ;
  XSetIconName(mydisplay, mywindow, nombre.c_str()) ;
  mygc = XCreateGC(mydisplay, mywindow, 0 ,0) ;
  XSelectInput(mydisplay, mywindow,ExposureMask | ButtonPressMask) ;   
  XMapRaised(mydisplay, mywindow) ;
  XFlush(mydisplay) ;

  // initialize text

  //  char* font_name = "*-fixed-medium-r-normal*-13-*";
  char* font_name = "*-helvetica-medium-r-normal*-12-*";
  font_info = XLoadQueryFont(mydisplay, font_name);
  if (!font_info) {
    cout << "XLoadQueryFont: failed loading font " << font_name << endl;
  }
  XSetFont(mydisplay, mygc, font_info->fid);


  // initialize color table
  // ----------------------

  Colormap cmap= DefaultColormap( mydisplay,  myscreen );
  XColor mycolor;
  mycolor.red  = 0;
  mycolor.green= 0;
  mycolor.blue = 0;
  XAllocColor(mydisplay, cmap, &mycolor);
  colorbase    = mycolor.pixel;
  mycolor.red  = 65535;
  XAllocColor(mydisplay, cmap, &mycolor);
  colorred     = mycolor.pixel;
  mycolor.red  = 0;
  mycolor.green= 65535;
  XAllocColor(mydisplay, cmap, &mycolor);
  colorgre     = mycolor.pixel;
  mycolor.green= 0;
  mycolor.blue = 65535;
  XAllocColor(mydisplay, cmap, &mycolor);
  colorblu     = mycolor.pixel;

  winID = mywindow;
}



void IgluDriverX::color_set_this_driver(double r1,double g1,double b1){
  Colormap cmap= DefaultColormap( mydisplay,  myscreen );
  XColor mycolor;
  mycolor.red =  (fabs(r1)>1)? 65535: int(65535*fabs(r1)) ;
  mycolor.green= (fabs(g1)>1)? 65535: int(65535*fabs(g1)) ;
  mycolor.blue=  (fabs(b1)>1)? 65535: int(65535*fabs(b1)) ;
  XAllocColor(mydisplay, cmap, &mycolor);
  XSetForeground(mydisplay, mygc, mycolor.pixel );
}

void IgluDriverX::scale_font_this_driver(double f){
  current_scale_font = f;
}

/*
// La eliminamos por ahora, ya veremos si conviene reponerla o no
void IgluDriverX::color_set(int i){
  if (i ==0) XSetForeground(mydisplay,mygc,background);
  if (i ==1) XSetForeground(mydisplay,mygc,foreground);
}
*/

void IgluDriverX::plot_line(double x1,double y1,double x2,double y2,
  IgluColor c){
  color_set(c);
  XDrawLine(mydisplay, mywindow,mygc,px(x1),py(y1),px(x2),py(y2));
} 


void IgluDriverX::plot_point(double x,double y,IgluColor c){
  color_set(c);
  XDrawPoint(mydisplay, mywindow, mygc, px(x), py(y));
}

void IgluDriverX::plot_rectangle(double x1,double y1,double x2,double y2,
				IgluColor c){
  color_set(c);
  XDrawRectangle(mydisplay,mywindow,mygc,
		 px(x1),py(y2),px(x2)-px(x1),py(y1)-py(y2));
}

void IgluDriverX::plot_filled_rectangle(double x1,double y1,double x2,
				       double y2,IgluColor c){
  color_set(c);
  XFillRectangle(mydisplay,mywindow,mygc,
		 px(x1),py(y2),px(x2)-px(x1),py(y1)-py(y2));
}

// void IgluDriverX::plot_arc_circle(double x,double y,double r,
// 				 double theta_i,double theta_f,
// 				 IgluColor c){
//   //  cout << "Dibujando arco circulo" << endl;
//   color_set(c);
//   /* Buscando solucionar el problema de circulos no
//      concentricos (ejemplo_iglu_19.cc) */
//   /*  cout << "X plot_arc_circle: " << endl;
//   cout << "   " << x << " " << y << " " << r << " " 
//        << theta_i << " " << theta_f << endl;
//   cout << "   " << px(x-r) << " " << py(y+r) << " " 
//        << px(x+2*r)-px(x) << " " << -py(y+2*r)+py(y) << " "
//        <<  int(theta_i*64) << " " << int(theta_f*64) << endl;
//   cout << "   " << px(x-r) << " " << py(y+r) << " " 
//        << px(x+r)-px(x-r) << " " << -py(y+r)+py(y-r) << " "
//        <<  int(theta_i*64) << " " << int(theta_f*64) << endl;*/
//   XDrawArc(mydisplay,mywindow,mygc,
//  	   px(x-r), py(y+r), px(x+r)-px(x-r), -py(y+r)+py(y-r), 
// 	   int(theta_i*64), int(theta_f*64));  
// }

void IgluDriverX::plot_arc_ellipse(double x,double y,double radio_x,
					  double radio_y,double theta_i,
					  double theta_f,IgluColor c){
  //  cout << "Dibujando arco elipse" << endl;
  color_set(c);
  XDrawArc(mydisplay,mywindow,mygc,
 	   px(x-radio_x), py(y+radio_y), 
	   px(x+radio_x)-px(x-radio_x), -py(y+radio_y)+py(y-radio_y), 
	   int(theta_i*64), int((theta_f-theta_i)*64));  
}

void IgluDriverX::plot_filled_arc_ellipse(double x,double y,double radio_x,
					  double radio_y,double theta_i,
					  double theta_f,IgluColor c){
  color_set(c);
  XFillArc(mydisplay,mywindow,mygc,
 	   px(x-radio_x), py(y+radio_y), px(x+radio_x)-px(x-radio_x),
	   -py(y+radio_y)+py(y-radio_y), 
	   int(theta_i*64), int((theta_f-theta_i)*64));  
}

void IgluDriverX::plot_text(double x, double y, string s, IgluColor c,
			      double scale){
  color_set(c);
  XDrawString(mydisplay, mywindow, mygc,px(x),py(y),s.c_str(),s.length());
}

  void IgluDriverX::plot_centered_text(double x, double y, string s, IgluColor c, double scale){
  int string_width = XTextWidth(font_info, s.c_str(), s.length());
  double x_corr = x - scale_point2x(string_width)/2;

  //  int font_height = font_info->ascent + font_info->descent;
  //  y_corr = (win_width - font_height) / 2;

  //  cout << x << " -> " << x_corr << endl;
  plot_text(x_corr,y,s,c,scale);
}

void IgluDriverX::plot_clip_rectangle(double xmin,double ymin,
  double xmax,double ymax,IgluColor c){

//   cout << xmax-xmin << " " << ymax - ymin << endl;
//   cout << "XRectangle: " << px(xmin) << " " << py(ymin) << " " 
//        << scale_point2px(xmax-xmin) << " " 
//        << scale_point2py(ymax-ymin) << endl;
  XRectangle rectangles[1] = {
    {px(xmin),py(ymax),scale_point2px(xmax-xmin),scale_point2py(ymax-ymin)}
  };
  XSetClipRectangles(mydisplay, mygc, 0, 0, rectangles, 
		     1, Unsorted);
}

//////////////////////////////////////////
// POSITIONS GRAPHS VARIABLES IN PIXELS
//////////////////////////////////////////

int IgluDriverX::px(double x){
  int rX=xiP+int((x-xiR)*double(anchoxP())/anchoxR());
  /*  if(rX<xiP) rX=xiP;
      if(rX>xfP) rX=xfP;*/
  return rX;
}

int IgluDriverX::py(double y){
  int rY=yiP-int((y-yiR)*double(anchoyP())/anchoyR());
  /*  if(rY<yfP) rY=yfP;
      if(rY>yiP) rY=yiP;*/
  return rY;
}

double IgluDriverX::scale_point2x(int n){
  return n*anchoxR()/double(anchoxP());
}

double IgluDriverX::scale_point2y(int n){
  return n*anchoyR()/double(anchoyP());
}

int IgluDriverX::scale_point2px(double x){
  return int((x/anchoxR())*double(anchoxP()));
}

int IgluDriverX::scale_point2py(double y){
  return int((y/anchoyR())*double(anchoyP()));
}

int IgluDriverX::anchoxP(){
  return xfP-xiP;
}

int IgluDriverX::anchoyP(){
  return yiP-yfP;
}

int IgluDriverX::xmin_pixeles(){
  return xiP;
}

int IgluDriverX::xmax_pixeles(){
  return xfP;
}

int IgluDriverX::ymin_pixeles(){
  return yiP;
}

int IgluDriverX::ymax_pixeles(){
  return yfP;
}

void IgluDriverX::flush(){
  XFlush(mydisplay);
}

void IgluDriverX::wait(vector<list<IgluObjetoColor *> > l){
  XSelectInput(mydisplay, mywindow,  ExposureMask | KeyPressMask | ButtonPressMask);
  XEvent myevent;
  int done = 0;
  while (done == 0) {
    XNextEvent(mydisplay, &myevent);
    switch(myevent.type) {
      case Expose:{
	/* Version VM */
	/* if this is the first in a set of expose events
	   - ignore this event.*/
	if (myevent.xexpose.count != 0)
	  break;
	update_size();
	replot(l);

      }
      break;
      case KeyPress:{
       /* translate the key code to a key symbol. */
        KeySym key_symbol = XKeycodeToKeysym(mydisplay, 
					     myevent.xkey.keycode, 0);
        switch (key_symbol) {
	case XK_Return:
	case XK_space:
	  done=1;
	  break;
	case XK_R:
	case XK_r:
	  restore_size();
	  replot(l);
	  break;
	}
      }
      break;
      case ButtonPress:{
	/*	if (myevent.xbutton.button==3) {
	    string s="menu.wish ";
	    s=s+dec(winID);
	    system(s.c_str());
	    }*/
      }
    }
  }
}

void IgluDriverX::wait(vector<list<IgluObjetoColor *> > l,double tval){
  int begin = clock(), end = clock(), duration = end - begin,
    tmax = int(tval*CLOCKS_PER_SEC), sec_fraction = CLOCKS_PER_SEC/10,
    bin_old = duration/sec_fraction, bin;
  
  while (duration < tmax){
    end = clock();
    duration = end - begin;
    bin = duration / sec_fraction;
    if (bin > bin_old){
      bin_old = bin;
      //      cout << double(bin_old)/10 << endl;
      replot(l);
    }
  }
  //  cout << double(tmax)/double(CLOCKS_PER_SEC) << endl;
  replot(l);
}

void IgluDriverX::clean(){
  XClearWindow(mydisplay,mywindow);
}

void IgluDriverX::update_size(){
  
  /* this variable will contain the attributes of the window. */
  XWindowAttributes win_attr;

  /* query the window's attributes. */
  Status rc = XGetWindowAttributes(mydisplay, mywindow, &win_attr);
  
  //  cout << win_attr.width << " x " << win_attr.height << " pixeles" << endl;
  xfP = win_attr.width;
  yiP = win_attr.height;
}

void IgluDriverX::restore_size(){
  xfP = ancho_default;
  yiP = alto_default;
  XResizeWindow(mydisplay, mywindow, xfP, yiP);
}

void IgluDriverX::replot(vector<list<IgluObjetoColor *> >l){
  plot_clip_rectangle(xiR,yiR,xfR,yfR,background_color());
  for (unsigned int i=0;i<l.size();i++){
    IgluDriver::replot(l[i]);
  }
}

IgluDriverPostScript::IgluDriverPostScript(int warn){
  variables_default();
  WARN = warn;
  /* crear_dibujo() va al llamar set_variables() */
}


IgluDriverPostScript::IgluDriverPostScript(string s){
  variables_default();
  nombre = s;
  /* crear_dibujo() va al llamar set_variables() */
}

IgluDriverPostScript::~IgluDriverPostScript(){
  //cout << "Destructor llamado" << endl;
  archivo << "S showpage" << endl;
  archivo.close();
}

void IgluDriverPostScript::bounding_box(double ancho,double alto){
  if (ancho > alto){
      ancho_bounding_box = BOUNDING_BOX_MAX;
      alto_bounding_box = alto*(ancho_bounding_box/ancho);
  }
  else {
    alto_bounding_box = BOUNDING_BOX_MAX;
    ancho_bounding_box = ancho*(alto_bounding_box/alto);
  }

//   cout << "bbox: " << ancho << " " << alto << endl;
//   cout << "bbox: " << BOUNDING_BOX_MAX << endl;
//   cout << "bbox: " << ancho_bounding_box << " " << alto_bounding_box << endl;

}

void IgluDriverPostScript::set_variables(int ancho_puntos,int alto_puntos,
					 string n){
  xiP = 0;
  xfP = ancho_puntos;
  yiP = 0;
  yfP = alto_puntos;
  xiR = xiP;
  xfR = xfP;
  yiR = yiP;
  yfR = yfP;
  nombre = n;
  crear_dibujo();
}

void IgluDriverPostScript::variables_default(){
  current_color = IgluColor(0,0,0);
  current_background_color = IgluColor(1,1,1);
  current_scale_font = 1;
  nombre = "dibujo";
  WARN = 0;
  BOUNDING_BOX_MAX = 500;
  ancho_bounding_box_default = 100;
  alto_bounding_box_default = 100;
  ancho_bounding_box = ancho_bounding_box_default;
  alto_bounding_box = alto_bounding_box_default;
}

void IgluDriverPostScript::crear_dibujo(){
  /* Abrir archivo, verificar que no existe 
     Se cierra en el destructor */
  if (WARN==1){
    archivo.open(nombre.c_str(),ios::in);
    int apendice = 0;
    string nombre_base = nombre;
    while (!archivo.fail()){
      apendice++;
      stringstream temp;
      temp << apendice;
      string nombre_old = nombre;
      nombre = nombre_base + "." + temp.str();
      archivo.close();
      cout << "Iglu: " << nombre_old.c_str() << " existe. Cambiando nombre a: "
	   << nombre.c_str() << endl; 
      archivo.open(nombre.c_str(),ios::in);
    }
    archivo.close();
  }
  archivo.open(nombre.c_str(),ios::out);
    
  /* Tama~no pagina letter: 0 0 612 790 ~ 0 0 610 790*/

  double origen_bounding_box_x = (610 - ancho_bounding_box)/2;
  double origen_bounding_box_y = (790 - alto_bounding_box)/2;
    
  cout << "crear_dibujo:current_scale_font:" << current_scale_font << endl;

  archivo << "%!PS-Adobe-2.0 EPSF-2.0" << endl
	  << "%%Title: " << nombre << endl
	  << "%%Creator: Iglu" << endl
    //	  << "%%CreationDate: Fri Oct 18 00:04:37 2002" << endl
    //	  << "%%BoundingBox: " << xiP << " " << yiP << " " 
    //	  << xfP << " " << yfP << endl
	  << "%%BoundingBox: " << origen_bounding_box_x << " " 
	  << origen_bounding_box_y << " " 
	  << origen_bounding_box_x + ancho_bounding_box  << " " 
	  << origen_bounding_box_y + alto_bounding_box << endl
	  << "%%EndComments" << endl
	  << "/SC {setrgbcolor} def" << endl
	  << "/M {moveto} def" << endl
	  << "/L {lineto} def" << endl
	  << "/S {stroke} def" << endl
	  << "/A {arc} def" << endl
	  << "/CP {closepath} def" << endl
	  << "/F {fill} def" << endl
	  << "/N {newpath} def" << endl
	  << "/ShowCentered {dup stringwidth pop 2 div neg 0 rmoveto show} def" << endl
	  << "/Helvetica findfont " << current_scale_font*10 << " scalefont setfont" << endl
	  << origen_bounding_box_x << " " << origen_bounding_box_y
	  << " translate" << endl
	  << "0 0 M" << endl
	  << "% " << ancho_bounding_box << " " << xfP << " " << xiP 
	  << " " << alto_bounding_box << " " << yfP << " " << yiP << endl
	  << ancho_bounding_box/(xfP-xiP) << " " 
	  << alto_bounding_box/(yfP-yiP) << " scale" << endl;
}

void IgluDriverPostScript::color_set_this_driver(double r1,double g1,
						 double b1){
  archivo << r1 << " " << g1 << " " << b1 << " SC" << endl;
}

void IgluDriverPostScript::scale_font_this_driver(double f){
  current_scale_font = f;
}

void IgluDriverPostScript::plot_line(double x1,double y1,double x2,double y2,
			   IgluColor c){
  //  archivo << "%linea" << endl;
  color_set(c);
  archivo << "N" << endl
	  << px(x1) << " " << py(y1) << " M" << endl
	  << px(x2) << " " << py(y2) << " L" << endl
	  << "S" << endl;
} 

void IgluDriverPostScript::plot_point(double x,double y,IgluColor c){
  //  archivo << "%punto" << endl;
  color_set(c);
  archivo << "N" << endl
	  << px(x)   << " " << py(y)   << " M" << endl
	  << px(x)   << " " << py(y)+1 << " L" << endl
	  << px(x)+1 << " " << py(y)+1 << " L" << endl
	  << px(x)+1 << " " << py(y)   << " L" << endl
	  << px(x)   << " " << py(y)   << " L" << endl
	  << "S" << endl;
}

void IgluDriverPostScript::plot_rectangle(double x1,double y1,double x2,double y2,
				IgluColor c){
  //  archivo << "%rectangulo" << endl;
  color_set(c);
  archivo << "N" << endl
	  <<px(x1) << " " << py(y1) << " M" << endl
	  << px(x1) << " " << py(y2) << " L" << endl
	  << px(x2) << " " << py(y2) << " L" << endl
	  << px(x2) << " " << py(y1) << " L" << endl
	  << px(x1) << " " << py(y1) << " L" << endl
	  << "S" << endl;
}

void IgluDriverPostScript::plot_filled_rectangle(double x1,double y1,double x2,
				       double y2,IgluColor c){
  //  archivo << "%rectangulo lleno" << endl;
  color_set(c);
  archivo << "N" << endl
	  << px(x1) << " " << py(y1) << " M" << endl
	  << px(x1) << " " << py(y2) << " L" << endl
	  << px(x2) << " " << py(y2) << " L" << endl
	  << px(x2) << " " << py(y1) << " L" << endl
	  << px(x1) << " " << py(y1) << " L" << endl
	  << "CP F S" << endl;

}

void IgluDriverPostScript::plot_arc_ellipse(double x,double y,
					    double radio_x,double radio_y,
					    double theta_i,double theta_f,
					    IgluColor c){
  int flag;
  if (radio_x > radio_y){
    flag = 1;
  }
  else {
    flag = -1;
  }

  color_set(c);
  // Valido cuando no sabiamos que dibujar elipses es la solucion
  // para que los circulos deformados por map_coordinates se vean
  // como elipses
//   if (flag==0){
//     archivo << "N " << px(x) << " " << py(y) << " " 
// 	    << scale_point2px(radio_x) << " " 
// 	    << theta_i << " " << theta_f << " A S" << endl;
//   }
//   else{
  archivo << "%radios x,y: " << radio_x << "," << radio_y << endl;
  archivo << "gsave" << endl
	  << px(x) << " " << py(y) << " translate" << endl;
  if (flag==1){
    archivo << "1 " << scale_point2py(radio_y) << " " 
	    << scale_point2px(radio_x) << " div scale" << endl
	    << "N 0 0 " << scale_point2px(radio_x) << " " 
	    << theta_i << " " << theta_f << " A S" << endl;
  }
  else if (flag==-1){
    if (radio_x==0 && radio_y==0){
      archivo << "1 1 scale" << endl;
    }
    else {
    archivo << scale_point2px(radio_x) << " " 
	    << scale_point2py(radio_y) << " div 1 scale" << endl;
    }
    archivo << "N 0 0 " << scale_point2py(radio_y) << " " 
	    << theta_i << " " << theta_f << " A S" << endl;
  }


  archivo << "grestore" << endl; 

// gsave 
// 250 250 translate
// 1 50 100 div scale
// N 0 0 100 0 360 A S
// grestore
}

void IgluDriverPostScript::plot_filled_arc_ellipse(double x,double y,
						   double radio_x,
						   double radio_y,
						   double theta_i,
						   double theta_f,
						   IgluColor c){
  //  archivo << "%arco lleno" << endl;
  int flag;

  if (radio_x > radio_y){
    flag = 1;
  }
  else {
    flag = -1;
  }
 
  color_set(c);
  archivo << "gsave" << endl
	  << px(x) << " " << py(y) << " translate" << endl;
  if (flag==1){
    archivo << "1 " << scale_point2py(radio_y) << " " 
	    << scale_point2px(radio_x) << " div scale" << endl
	    << "N 0 0 M" << endl
	    << "0 0 " << scale_point2px(radio_x) << " " 
	    << theta_i << " " << theta_f << " A CP F S" << endl;
  }
  else if (flag==-1){
    if (radio_x==0 && radio_y==0){
      archivo << "1 1 scale" << endl;
    }
    else {
    archivo << scale_point2px(radio_x) << " " 
	    << scale_point2py(radio_y) << " div 1 scale" << endl;
    }

    archivo << "N 0 0 M" << endl
	    << "0 0 " << scale_point2py(radio_y) << " " 
	    << theta_i << " " << theta_f << " A CP F S" << endl;
  }


  archivo << "grestore" << endl; 
}

  void IgluDriverPostScript::plot_text(double x,double y,string s,IgluColor c,double scale){
  color_set(c);
  archivo  << "/Helvetica findfont " << scale*10 << " scalefont setfont" << endl;
  archivo << "N " << px(x) << " " << py(y) << " M (" << s << ") show" << endl;
}

  void IgluDriverPostScript::plot_centered_text(double x,double y,string s,IgluColor c,double scale){
  color_set(c);
  archivo  << "/Helvetica findfont " << scale*10 << " scalefont setfont" << endl;
  archivo << "N " << px(x) << " " << py(y) << " M (" << s 
	  << ") ShowCentered" << endl;
}

void IgluDriverPostScript::plot_clip_rectangle(double xmin,double ymin,
				      double xmax,double ymax,IgluColor c){
  archivo << "N " << endl
	  << px(xmin) << " " << py(ymin) << " M" << endl
	  << px(xmin) << " " << py(ymax) << " L" << endl
	  << px(xmax) << " " << py(ymax) << " L" << endl
	  << px(xmax) << " " << py(ymin) << " L" << endl
	  << px(xmin) << " " << py(ymin) << " L" << endl
	  << "closepath clip" << endl;
}

//////////////////////////////////////////
// POSITIONS GRAPHS VARIABLES IN "PIXELS"
//////////////////////////////////////////

double IgluDriverPostScript::scale_point2px(double x){
  return (x/anchoxR())*double(anchoxP());
}

double IgluDriverPostScript::scale_point2py(double y){
  return (y/anchoyR())*double(anchoyP());
}

double IgluDriverPostScript::px(double x){
  double rX=xiP+(x-xiR)*anchoxP()/anchoxR();
  /*  if(rX<xiP) rX=xiP;
      if(rX>xfP) rX=xfP;*/
  return rX;
}

double IgluDriverPostScript::py(double y){
  double rY=yiP+(y-yiR)*anchoyP()/anchoyR();
  /*  if(rY<yiP) rY=yiP;
      if(rY>yfP) rY=yfP;*/
  return rY;
}

double IgluDriverPostScript::anchoxP(){
  return xfP-xiP;
}

double IgluDriverPostScript::anchoyP(){
  return yfP-yiP;
}

}
