
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#ifndef IGLU_DRIVER
#define IGLU_DRIVER

#include <string>
//#include <stream.h>
#include <fstream>
#include <list>
#include <cstdlib>
#include <time.h>
#include <sstream>
#include <vector>
#include <cmath>
#include "Xlib.h"
#include "Xutil.h"
#include "keysym.h"
#include "iglu_color.h"
#include "iglu_objeto_color.h"
#include "iglu_linea.h"
//#include "iglu_clip.h"

namespace iglu{

  using namespace std;

class IgluDriver{
 protected:
  IgluColor current_color;
  IgluColor current_background_color;
  double current_scale_font;
  string nombre; // Nombre dibujo
  double xiR; // xmin en coordenadas reales
  double xfR; // etc.
  double yiR; 
  double yfR;
  virtual void color_set_this_driver(double,double,double) = 0;
  virtual void scale_font_this_driver(double) = 0;
  virtual void variables_default() = 0;
 public:
  IgluDriver();
  virtual ~IgluDriver();

  void map_coordinates(double,double,double,double);
  IgluColor foreground_color();
  IgluColor background_color();
  double anchoxR();
  double anchoyR();
  void color_set(double,double,double);
  //  void color_set(int); /* La vamos a eliminar por ahora */
  void color_set(IgluColor);
  void scale_font(double);
  double get_scale_font();
  virtual void set_variables(int,int,string="") = 0;

  virtual void plot_line(double,double,double,double,IgluColor) = 0;
  virtual void plot_point(double,double,IgluColor) = 0;
  virtual void plot_rectangle(double,double,double,double,IgluColor) = 0;
  virtual void 
    plot_filled_rectangle(double,double,double,double,IgluColor) = 0;
  virtual void 
    plot_arc_ellipse(double,double,double,double,double,double,IgluColor) = 0;
  virtual void 
    plot_filled_arc_ellipse(double,double,double,double,double,double,
			    IgluColor) = 0;
  virtual void plot_text(double,double,string,IgluColor,double) = 0;
  virtual void plot_centered_text(double,double,string,IgluColor,double) = 0;
  virtual void plot_clip_rectangle(double,double,double,double,IgluColor) = 0;


  //  void plot_line(IgluLinea *);

  double xmin_reales();
  double xmax_reales();
  double ymin_reales();
  double ymax_reales();
  string nombre_dibujo();

  void replot(list<IgluObjetoColor *> );
  void replot(vector<list<IgluObjetoColor *> >);
};

class IgluDriverX : public IgluDriver{
 private:

  Display *mydisplay;
  int myscreen;
  Window mywindow;
  GC mygc;
  int winID;
  XFontStruct* font_info;

  unsigned long background;
  unsigned long foreground;
  unsigned long colorbase;
  unsigned long colorred;
  unsigned long colorgre;
  unsigned long colorblu;
  unsigned long colorpixel;


  unsigned int border_width;

  int ancho_default; // Valores con los cuales fue construida
  int alto_default;

  int xiP;     // xmin en pixeles
  int xfP;      // etc.
  int yiP;
  int yfP;
  int xpos;      // Posicion horizontal
  int ypos;      // Posicion vertical

  int px(double);
  int py(double);
  double scale_point2x(int);
  double scale_point2y(int);
  int scale_point2px(double);
  int scale_point2py(double);
  int anchoxP();
  int anchoyP();
  void update_size();
  void restore_size();

  void variables_default();
  void crear_dibujo();
  void color_set_this_driver(double,double,double);
  void scale_font_this_driver(double);
 public: 
  IgluDriverX();
  ~IgluDriverX();

  void set_variables(int,int,string="");

  void plot_line(double,double,double,double,IgluColor);
  void plot_point(double,double,IgluColor);
  void plot_rectangle(double,double,double,double,IgluColor);
  void plot_filled_rectangle(double,double,double,double,IgluColor);
  void plot_arc_ellipse(double,double,double,double,double,double,IgluColor);
  void plot_filled_arc_ellipse(double,double,double,double,double,double,
			       IgluColor);
  void plot_text(double,double,string,IgluColor,double);
  void plot_centered_text(double,double,string,IgluColor,double);
  void plot_clip_rectangle(double,double,double,double,IgluColor);
 
  //void plot_line(IgluLinea);

  int xmin_pixeles();
  int xmax_pixeles();
  int ymin_pixeles();
  int ymax_pixeles();


  void wait(vector<list<IgluObjetoColor *> > );
  void wait(vector<list<IgluObjetoColor *> > ,double);
  void clean();
  void clear();
  void flush();
  void replot(vector<list<IgluObjetoColor *> > );
};

class IgluDriverPostScript : public IgluDriver{
 private:
  void variables_default();
  void crear_dibujo();
  void color_set_this_driver(double,double,double);
  void scale_font_this_driver(double);
  fstream archivo;

  int WARN;        // 0 : no revisa existencia de archivo
                   // 1 : revisa
  double BOUNDING_BOX_MAX; // Maximo ancho/alto permitido de bounding box
                          // 500 para letter.

  double xiP;      // xmin en "pixeles" (copiado de DriverX)
  double xfP;      // etc.
  double yiP;
  double yfP;

  double ancho_bounding_box_default;
  double alto_bounding_box_default;
  double ancho_bounding_box;
  double alto_bounding_box;

  double scale_point2px(double);
  double scale_point2py(double);
  double px(double);
  double py(double);
  double anchoxP();
  double anchoyP();

 public:
  IgluDriverPostScript(int=0);
  ~IgluDriverPostScript();
  IgluDriverPostScript(string);

  void bounding_box(double,double);
  void set_variables(int,int,string="dibujo");

  void plot_line(double,double,double,double,IgluColor);
  void plot_point(double,double,IgluColor);
  void plot_rectangle(double,double,double,double,IgluColor);
  void plot_filled_rectangle(double,double,double,double,IgluColor);
  void plot_arc_ellipse(double,double,double,double,double,double,IgluColor);  
  void plot_filled_arc_ellipse(double,double,double,double,double,double,
			       IgluColor);
  void plot_text(double,double,string,IgluColor,double); 
  void plot_centered_text(double,double,string,IgluColor,double); 
  void plot_clip_rectangle(double,double,double,double,IgluColor);
};

}

#endif
