/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include "iglu_grafico.h"

namespace iglu{

  using namespace std;

/* 640x450 es el default de Gnuplot, razon de 450/640 = .70312 */
IgluGrafico::IgluGrafico():IgluDibujo(710,500,"Iglu"){
  margin = .075;
  map_coordinates(-margin,1+margin,-margin,1+margin);
  set_layers(3);
  axes();
  radius_symbol_circle = .005;
  symbol_size_factor = 1;
  bar_space_factor = 0;
  current_line_style = IgluEstiloLinea(IgluColor(0,0,0),LINE_SOLID);
  current_point_style = IgluEstiloPunto(IgluColor(0,0,0),SYMBOL_NONE,
					symbol_size_factor);
  current_bar_style = IgluEstiloBarra(IgluColor(0,0,0),BAR_OPEN,
				      bar_space_factor);
  abscisa_dibujada = 0;
  axes_automatic_replot = AXES_FIX;
  frame_distance_factor = 0.1;
  current_axis_color = IgluColor(0,0,0);
}

IgluGrafico::~IgluGrafico(){
}

vector<double> IgluGrafico::origin(){
  vector<double> orig(2);
  orig[0] = (-x_axis_min)/(x_axis_max-x_axis_min);
  orig[1] = (-y_axis_min)/(y_axis_max-y_axis_min);
  return orig;
}

void IgluGrafico::axes(){
  x_axis_min = 0;
  x_axis_max = 1;
  y_axis_min = 0;
  y_axis_max = 1;

  use_layer(IGLU_LAYER_CLIP);
  clip_rectangle(0,0,1,1);
  use_layer(IGLU_LAYER_AXIS);
  IgluDibujo::color_set(current_axis_color);
  //  cout << "axes: " << current_axis_color << endl;
  IgluDibujo::plot_rectangle(0,0,1,1);
  title("");
  x_title("x");
  y_title("y");
  plot_text(0,-margin/2.0,"0","xmin");
  plot_text(1,-margin/2.0,"1","xmax");
  plot_text(-margin/2.0,0,"0","ymin");
  plot_text(-margin/2.0,1,"1","ymax");
  use_layer(IGLU_LAYER_DATA);
}

void IgluGrafico::axes(int flag){
  axes_automatic_replot = flag;
}

void IgluGrafico::axes(double xmin, double xmax, double ymin, double ymax){
  x_axis_min = xmin;
  x_axis_max = xmax;
  y_axis_min = ymin;
  y_axis_max = ymax;

  int old_layer = current_layer;
  use_layer(IGLU_LAYER_AXIS);
  stringstream temp[4];
  temp[0] << xmin;
  temp[1] << xmax;
  temp[2] << ymin;
  temp[3] << ymax;

  IgluDibujo::color_set(current_axis_color);
  erase_label("xmin");
  plot_text(0,-margin/2.0,temp[0].str(),"xmin");
  erase_label("xmax");
  plot_text(1,-margin/2.0,temp[1].str(),"xmax");
  erase_label("ymin");
  plot_text(-margin/2.0,0,temp[2].str(),"ymin");
  erase_label("ymax");
  plot_text(-margin/2.0,1,temp[3].str(),"ymax");
  plot_objetos_grafico();
  use_layer(old_layer);
}

void IgluGrafico::title(string t){
  int old_layer = current_layer;
  use_layer(IGLU_LAYER_AXIS);
  erase_label("title");
  plot_centered_text(.5,1+margin/2.0,t,"title");
  use_layer(old_layer);
}

void IgluGrafico::x_title(string t){
  int old_layer = current_layer;
  use_layer(IGLU_LAYER_AXIS);
  erase_label("x_title");
  plot_centered_text(.5,-margin/2.0,t,"x_title");
  use_layer(old_layer);
}

void IgluGrafico::y_title(string t){
  int old_layer = current_layer;
  use_layer(IGLU_LAYER_AXIS);
  erase_label("y_title");
  plot_centered_text(-margin/2.0,.5,t,"y_title");
  use_layer(old_layer);
}

void IgluGrafico::plot(double x,double y){
  vector<double> lista_x(1,x),lista_y(1,y);
  plot(lista_x,lista_y);
  /*  cout << lista_x[0] << " " << lista_y[0] << endl;
      exit(0);*/
}
  
void IgluGrafico::plot(double x[],double y[],int n){
  vector<double> lista_x(n), lista_y(n);
  for (int i=0;i<n;i++){
    lista_x[i] = x[i];
    lista_y[i] = y[i];
  }
  plot(lista_x,lista_y);
}

void IgluGrafico::plot(vector<double> x, vector<double> y){
  nuevo_objeto_grafico(x,y,"line");
}

void IgluGrafico::plot(vector<double> y){
  vector<double> x(y.size());
  for(unsigned int i=0;i<y.size();i++){
    x[i]=i;
  }
  nuevo_objeto_grafico(x,y,"line");
}

void IgluGrafico::plot_line(double x1,double y1,double x2,double y2){
  double X[2] = {x1,x2}, Y[2] = {y1,y2};
  plot(X,Y,2);
}  

void IgluGrafico::plot_rectangle(double xmin,double ymin,
				 double xmax,double ymax){

  double X[5] = {xmin,xmin,xmax,xmax,xmin};
  double Y[5] = {ymin,ymax,ymax,ymin,ymin};
  plot(X,Y,5);
    
}

  
void IgluGrafico::bar(double x[],double y[],int n){
  vector<double> lista_x(n), lista_y(n);
  for (int i=0;i<n;i++){
    lista_x[i] = x[i];
    lista_y[i] = y[i];
  }
  bar(lista_x,lista_y);
}

void IgluGrafico::bar(vector<double> x, vector<double> y){
  nuevo_objeto_grafico(x,y,"bar");

//   double dx = (1-bar_space_factor)*(x[1]-x[0])/2.0;
//   for (int i=0;i<int(x.size());i++){
//   cout<< "b: hola" << endl;
//     plot_rectangle(x[i]-dx,0,x[i]+dx,y[i]);
//   cout<< "b: hola" << endl;
//   }
}

void IgluGrafico::nuevo_objeto_grafico(vector<double> x,vector<double> y,
				       string s){
  IgluDatosGrafico * d;
  //  cout << "nuevo: " << s << endl;
  
  if (s=="line"){
    d = new IgluLineaGrafico(x,y,current_line_style,
			     current_point_style);
  }
  else if (s=="bar"){
    d = new IgluBarrasGrafico(x,y,current_bar_style);
    //    dibujar_abscisa();
  }

  recalcular_ejes(x,y);
  lista_objetos_grafico.push_back(d);
  //  lista_objetos_grafico.insert_after(lista_objetos_grafico.previous(lista_objetos_grafico.end()),d);
  plot_objetos_grafico(d);
}

void IgluGrafico::plot_objetos_grafico(IgluDatosGrafico * l){
  //  cout << "color linea: " << l->style().color() << endl;
  vector<double> x_plot_data = l->x(), y_plot_data = l->y();
  /*  cout << "plot_objetos_grafico(l)" << endl;
  cout << " " << x_plot_data[0] << " " << x_plot_data[1] << endl;
  cout << " " << y_plot_data[0] << " " << y_plot_data[1] << endl;*/

  int n = l->x().size();
  vector<double> x_data(n),y_data(n), origen = origin();
  for (int i=0;i<n;i++){
    x_data[i] = (l->x(i)-x_axis_min)/(x_axis_max-x_axis_min);
    y_data[i] = (l->y(i)-y_axis_min)/(y_axis_max-y_axis_min);
  }
  if (l->tipo()==LINE_DATA){
    double radio = radius_symbol_circle*(l->point_style().size_factor());
    switch(l->line_style().line()){
    case LINE_NONE:
      break;
    case LINE_SOLID:
      IgluDibujo::color_set(l->line_style().color());
      IgluDibujo::plot_line(x_data,y_data);
      IgluDibujo::color_set(0,0,0);
      break;
    }
    
    switch(l->point_style().symbol()){
    case SYMBOL_NONE:
      break;
    case SYMBOL_DOT:
      IgluDibujo::color_set(l->point_style().color());
      plot_point(x_data,y_data);
      IgluDibujo::color_set(0,0,0);
      break;
    case SYMBOL_CIRCLE:
      IgluDibujo::color_set(l->point_style().color());
      for (unsigned int i=0;i<x_data.size();i++){
	plot_fake_circle(x_data[i],y_data[i],radio,"x");
      }
      IgluDibujo::color_set(0,0,0);
      break;
    case SYMBOL_FILLED_CIRCLE:
      IgluDibujo::color_set(l->point_style().color());
      for (unsigned int i=0;i<x_data.size();i++){
	plot_fake_filled_circle(x_data[i],y_data[i],radio,"x");
      }
      IgluDibujo::color_set(0,0,0);
      break;
    }
  }
  else if (l->tipo()==BAR_DATA){
    IgluDibujo::color_set(l->bar_style().color());
    double dx = (1-l->bar_style().space_factor())*(x_data[1]-x_data[0])/2.0;
    vector<double> yrange(2);
    for (int i=0;i<int(x_data.size());i++){
      switch(l->bar_style().bar_style()){
      case BAR_NONE:
	break;
      case BAR_OPEN:
	if (l->y(i)>=0){
	  yrange[0] = origen[1];
	  yrange[1] = y_data[i];
	}
	else {
	  yrange[0] = y_data[i];
	  yrange[1] = origen[1];
	}
	IgluDibujo::plot_rectangle(x_data[i]-dx,yrange[0],
				   x_data[i]+dx,yrange[1]);
 	break;
      case BAR_FILLED:
	if (l->y(i)>=0){
	  yrange[0] = origen[1];
	  yrange[1] = y_data[i];
	}
	else {
	  yrange[0] = y_data[i];
	  yrange[1] = origen[1];
	}
	IgluDibujo::plot_filled_rectangle(x_data[i]-dx,yrange[0],
					  x_data[i]+dx,yrange[1]);
 	break;
      }
    }
    IgluDibujo::color_set(0,0,0);
  }
}

void IgluGrafico::plot_objetos_grafico(){
  clear_layer(IGLU_LAYER_DATA);
  //  report_status();
  use_layer(IGLU_LAYER_DATA);
  for (list<IgluDatosGrafico *>::iterator it = 
	 lista_objetos_grafico.begin();
       it != lista_objetos_grafico.end(); it++){
    plot_objetos_grafico((*it));
  }
  replot();
}

void IgluGrafico::dibujar_abscisa(){
  if (!abscisa_dibujada){
    plot_line(x_axis_min,0,x_axis_max,0);
    abscisa_dibujada = 1;
  }
}

void IgluGrafico::recalcular_ejes(vector<double> x,vector<double> y){

  if (axes_automatic_replot==AXES_AUTO){

    double xmin_data = *min_element(x.begin(),x.end());
    double xmax_data = *max_element(x.begin(),x.end());
    double ymin_data = *min_element(y.begin(),y.end());
    double ymax_data = *max_element(y.begin(),y.end());
    double xmin_temp=0, xmax_temp=0, ymin_temp=0, ymax_temp=0;

    int recalculado = 0; 
    //    cout << "vacio: " << lista_objetos_grafico.empty() << endl;
    if (lista_objetos_grafico.empty()){
      xmin_temp = xmin_data - (xmax_temp-xmin_data)*frame_distance_factor;
      xmax_temp = xmax_data + (xmax_data-xmin_temp)*frame_distance_factor;
      ymin_temp = ymin_data - (ymax_temp-ymin_data)*frame_distance_factor;
      ymax_temp = ymax_data + (ymax_data-ymin_temp)*frame_distance_factor;
      recalculado = 1;
    }
    else {
      xmin_temp=x_axis_min;
      xmax_temp=x_axis_max;
      ymin_temp=y_axis_min;
      ymax_temp=y_axis_max;

      if (xmin_data < x_axis_min){
	xmin_temp = xmin_data - (xmax_temp-xmin_data)*frame_distance_factor;
	recalculado = 1;
      }
      if (xmax_data > x_axis_max){
	xmax_temp = xmax_data + (xmax_data-xmin_temp)*frame_distance_factor;
	recalculado = 1;
      }
      if (ymin_data < y_axis_min){
	ymin_temp = ymin_data - (ymax_temp-ymin_data)*frame_distance_factor;
	recalculado = 1;
      }
      if (ymax_data > y_axis_max){
	ymax_temp = ymax_data + (ymax_data-ymin_temp)*frame_distance_factor;
	recalculado = 1;
      }
    }
    
    if (recalculado==1){
      //      cout << "datos-> [" << xmin_data << " " << xmax_data << " " << ymin_data << " " << ymax_data << "]" << endl;
      //       cout << "ejes-> [" << xmin_temp << " " << xmax_temp << " " << ymin_temp << " " << ymax_temp << "]" << endl;

      renormalize_axis(xmin_temp,xmax_temp);
      renormalize_axis(ymin_temp,ymax_temp);
      axes(xmin_temp,xmax_temp,ymin_temp,ymax_temp);
    }
  }
}

  void IgluGrafico::renormalize_axis(double& min,double& max){
    double tick_space = tick_spacing(min,max);
    //    cout << "tick: " << tick_space << endl;
    double min_val = tick_space * floor(min / tick_space), 
      max_val=min_val+tick_space;
    while(max_val<=max){
      max_val+=tick_space;
    }
    //    cout << min_val << "," << max_val << endl;
    min = min_val;
    max = max_val;
  }

  double IgluGrafico::tick_spacing(double min,double max){
    /*From PlPlot package

    file: src/pldtik.c
    function: void pldtik();

    Copyright (C) 2004  Alan W. Irwin
    */

/*----------------------------------------------------------------------*\
 * void pldtik() 
 *
 * Determine tick spacing: works out a "nice" interval (if tick == 0) such
 * that there are between 3 and 7.5 major tick intervals in the input
 * range vmin to vmax.  The recommended number of subticks is returned in
 * "nsubt" unless the routine is entered with a non-zero value of "nsubt".
 * n.b. big change: now returns only positive values of tick and nsubt
\*----------------------------------------------------------------------*/

    double vmin = min, vmax=max, tick=0;
    int nsubt=0;
    double t1, t2, tick_reasonable;
    int np, ns;

    /* Magnitude of min/max difference to get tick spacing */

    t1 = log10(fabs(vmax - vmin));
    np = int(floor(t1));
    t1 = t1 - np;

    /* Get tick spacing. */

    if (t1 > 0.7781512503) {
	t2 = 2.0;
	ns = 4;
    }
    else if (t1 > 0.4771212549) {
	t2 = 1.0;
	ns = 5;
    }
    else if (t1 > 0.1760912591) {
	t2 = 5.0;
	ns = 5;
	np = np - 1;
    }
    else {
	t2 = 2.0;
	ns = 4;
	np = np - 1;
    }

    /* Now compute reasonable tick spacing */

    tick_reasonable = t2 * pow(10.0, double(np));
    if (tick == 0) {
	tick = t2 * pow(10.0, double(np));
    }
    else {
      /*Our version does not permit specify tick spacing yet*/
      //        tick = fabs(tick);
      // if(tick < 1.e-4*tick_reasonable) {
      //  plexit("pldtik: magnitude of specified tick spacing is much too small");
      //  return;
      //	}
    }
    if (nsubt == 0){
      nsubt = ns;
    }

    nsubt = abs(nsubt);
    //    cout << "tick: " <<  min << "-" << max << "->" << tick  << endl;
    return tick;
      
  }

void IgluGrafico::line_color_set(double r,double g,double b){
  current_line_style.color_set(r,g,b);
  //  cout << "estilo actual: " << current_line_style << endl;
}

void IgluGrafico::symbol_color_set(double r,double g,double b){
  current_point_style.color_set(r,g,b);
}

void IgluGrafico::line_set(int n){
  current_line_style.line_set(n);
}

void IgluGrafico::symbol_set(int n){
  current_point_style.symbol_set(n);
}

void IgluGrafico::symbol_size_factor_set(double f){
  current_point_style.size_factor_set(f);
}

void IgluGrafico::bar_space_factor_set(double f){
  if (f>=0 & f<=1){
    current_bar_style.bar_space_factor_set(f);
  }
  else {
    cout << "IgluGrafico::bar_space_set(f) : " << endl
	 << "     f debe estar entre 0 y 1. Ignorando. " << endl;
  }
}

void IgluGrafico::bar_set(int n){
  current_bar_style.bar_style_set(n);
}

void IgluGrafico::bar_color_set(double r,double g,double b){
  current_bar_style.color_set(r,g,b);
}

  void IgluGrafico::clear_plot(){
    clear_layer(IGLU_LAYER_DATA);
    lista_objetos_grafico.clear();
    replot();
  }


void IgluGrafico::report_status(){
  IgluDibujo::report_status();
  
  cout << "     Objetos Grafico: " << endl;
  for (list<IgluDatosGrafico *>::iterator it = 
	 lista_objetos_grafico.begin();
       it != lista_objetos_grafico.end(); it++){
    (*it)->report_status();
  }
}

}
