
/* This file is part of Iglu

  Copyright (C) 2002,2003,2004 Jose Rogan, Xavier Andrade, 
                               Juan Alejandro Valdivia, Victor Munoz

  Iglu is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  Iglu is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Iglu; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#ifndef IGLU_GRAFICO
#define IGLU_GRAFICO

#include <string>
#include <vector>
#include <algorithm>
#include <iterator>
#include <list>
#include <cstdlib>
#include "iglu_dibujo.h"
#include "iglu_linea_grafico.h"
#include "iglu_barras_grafico.h"
//#include "iglu_rectangulo_grafico.h"
#include "iglu_estilo_linea.h"
#include "iglu_estilo_punto.h"
#include "iglu_estilo_barra.h"
#include "iglu_color.h"

namespace iglu{

  using namespace std;

enum {IGLU_CERO, IGLU_LAYER_AXIS, IGLU_LAYER_CLIP, IGLU_LAYER_DATA};
enum {AXES_FIX,AXES_AUTO};

class IgluGrafico : public IgluDibujo{
 private:
  double x_axis_min; /* Rango de los ejes x e y */
  double x_axis_max;
  double y_axis_min;
  double y_axis_max;
  int axes_automatic_replot; /* 0: ejes fijos
				1: ejes ajustan para mostrar todos los 
				   puntos */
  vector<double> origin(); 
                            /* Posicion del origen respecto al frame,
			       de ancho entre 0 y 1 y alto entre 0 y 1 */
  double margin;     /* Distancia entre ejes y borde de la ventana.
			(Porcentaje del ancho y del alto.) */
  //double xtick_space; /* Espacio entre ticks */
  //double ytick_space;

  list<IgluDatosGrafico *> lista_objetos_grafico;
  IgluColor current_axis_color;
  IgluEstiloLinea current_line_style;
  IgluEstiloPunto current_point_style;
  IgluEstiloBarra current_bar_style;
  void nuevo_objeto_grafico(vector<double>,vector<double>,string);
  void plot_objetos_grafico();
  void plot_objetos_grafico(IgluDatosGrafico *);
  void dibujar_abscisa();
  void recalcular_ejes(vector<double>,vector<double>);
  double radius_symbol_circle;
  double symbol_size_factor;
  double bar_space_factor;  /* Separacion barras en grafico de barras 
			     0: barras no separadas 
			     1: separacion igual a espaciado entre
			        abscisas, i.e., barras ancho cero */
  int abscisa_dibujada; /* 0 si la abscisa no esta aun dibujada explicitamente
			   (cuando y=0 no coincide con el lado inferior del
			   frame) */
  double frame_distance_factor; /* Factor de distancia entre el frame
				   y el borde (usado en recalcular_ejes) */
  double tick_spacing(double,double);
  void renormalize_axis(double&,double&);
 public: 
  IgluGrafico();
  ~IgluGrafico();
  void axes();
  void axes(int);
  void axes(double,double,double,double);
  void title(string);
  void x_title(string);
  void y_title(string);
  
  void plot(double,double);
  void plot(double [],double [],int);
  void plot(vector<double>,vector<double>);
  void plot(vector<double>);
  void plot_rectangle(double,double,double,double);
  void plot_line(double,double,double,double);
  void bar(double [],double [],int);
  void bar(vector<double>,vector<double>);

  void line_color_set(double,double,double);
  void symbol_color_set(double,double,double);
  void line_set(int);
  void symbol_set(int);
  void symbol_size_factor_set(double);
  void bar_space_factor_set(double);
  void bar_set(int);
  void bar_color_set(double,double,double);
  void clear_plot();

  void report_status();
};

}

#endif
